@@ -1,7 +1,10 @@
 """Instructions for the user"""
+import logging
+import os
+import textwrap
 from argparse import ArgumentDefaultsHelpFormatter
+
 from . import util, defs, config
-import os, textwrap, logging
 
 
 def _item(text):
@@ -17,7 +20,7 @@ def _print(text):
 def _print_s400_instructions(info):
     """Print instructions for configuration of the Novra S400
     """
-    util._print_header("Novra S400")
+    util.print_header("Novra S400")
 
     _print("""
     The Novra S400 is a standalone receiver, which will receive data from
@@ -25,22 +28,20 @@ def _print_s400_instructions(info):
     will need to configure both the S400 and the host.
     """)
 
-    util._print_sub_header("Connections")
+    util.print_sub_header("Connections")
 
     _print("The Novra S400 can be connected as follows:")
 
-
     print(("LNB ----> S400 (RF1 Interface) -- "
            "S400 (LAN 1 Interface) ----> Host / Network\n"))
 
-
     _item("Connect the LNB directly to interface RF1 of the S400 using a "
           "coaxial cable (an RG6 cable is recommended).")
     _item("Connect the S400's LAN1 interface to your computer or network.")
 
     util.prompt_for_enter()
 
-    util._print_sub_header("Network Connection")
+    util.print_sub_header("Network Connection")
 
     _print("Next, make sure the S400 receiver is reachable by the host.")
 
@@ -60,7 +61,7 @@ def _print_s400_instructions(info):
 
     util.prompt_for_enter()
 
-    util._print_sub_header("Software Requirements")
+    util.print_sub_header("Software Requirements")
 
     _print("Next, install all software pre-requisites on your host. Run:")
 
@@ -71,20 +72,20 @@ def _print_s400_instructions(info):
 
     util.prompt_for_enter()
 
-    util._print_sub_header("Receiver and Host Configuration")
+    util.print_sub_header("Receiver and Host Configuration")
 
     print("Now, configure the S400 receiver and the host by running:")
 
-    print("\n    sudo blocksat-cli standalone cfg\n")
+    print("\n    blocksat-cli standalone cfg\n")
 
     _print("""If you would like to review the changes that will be made to the
-    host before applying them, first run the command as a non-root user:""")
+    host before applying them, first run the command in dry-run mode:""")
 
-    print("    blocksat-cli standalone cfg\n\n")
+    print("    blocksat-cli standalone cfg --dry-run\n\n")
 
     util.prompt_for_enter()
 
-    util._print_sub_header("Monitoring")
+    util.print_sub_header("Monitoring")
 
     print("Finally, you can monitor your receiver by running:")
 
@@ -99,16 +100,19 @@ def _print_usb_rx_instructions(info):
 
     name = (info['setup']['vendor'] + " " + info['setup']['model']).strip()
 
-    util._print_header(name)
+    util.print_header(name)
 
     _print("""
-    The {0} is a USB receiver, which will receive data from satellite and
-    will output data to the host over USB. The host, in turn, is responsible for
-    configuring the receiver using specific DVB-S2 tools. Hence, next, you
-    need to prepare the host for driving the {0}.
+    The {0} is a USB-based DVB-S2 receiver. It receives the satellite signal
+    fed via a coaxial interface and outputs data to the host over USB. It is
+    also configured directly via USB, and the host is responsible for setting
+    such configurations using specific Linux tools.
     """.format(name))
 
-    util._print_sub_header("Hardware Connections")
+    _print("The instructions that follow prepare the host for driving the "
+           "{0} receiver.".format(name))
+
+    util.print_sub_header("Hardware Connections")
 
     print("The {} should be connected as follows:\n".format(name))
 
@@ -117,36 +121,38 @@ def _print_usb_rx_instructions(info):
 
     _item("Connect the LNB directly to \"LNB IN\" of the {} using a coaxial"
           " cable (an RG6 cable is recommended).".format(name))
-    _item("Connect the {}'s USB interface to your computer.".format(name))
+    _item("Connect the {}'s USB2.0 interface to your computer.".format(name))
+    if (info['setup']['model'] == "5520SE"):
+        power_up_str = "Connect both male connectors of the dual-male " + \
+            "USB Y cable to your host."
+    else:
+        power_up_str = "Connect the 12V DC power supply."
+    _item("Power up the {} device. {} ".format(name, power_up_str))
 
     util.prompt_for_enter()
 
-    util._print_sub_header("Drivers")
+    util.print_sub_header("Drivers")
 
     _print("""
-    Before anything else, note that specific device drivers are required in order to
-    use the {0}. Please, do note that driver installation can cause corruptions
-    and, therefore, it is safer and **strongly recommended** to use a virtual
-    machine for running the {0}. If you do so, please note that all commands
-    recommended in the remainder of this page are supposed to be executed in the
-    virtual machine.
+    Next, you will need to install specific device drivers to use the {0}.
+    These are installed by rebuilding and rewriting the Linux Media drivers.
+    Hence, if you are not setting up a dedicated machine to host the {0}, it
+    would be safer and recommended to use a virtual machine (VM) as the
+    receiver host so that the drivers can be installed directly on the VM
+    instead of your main machine.
     """.format(name))
 
     _print("Next, install the drivers for the {0} by running:".format(name))
 
-    print("""
-    blocksat-cli deps tbs-drivers
-    """)
+    print("    blocksat-cli deps tbs-drivers\n")
 
-    print("Once the script completes the installation, reboot the virtual "
-          "machine.")
+    print("Once the script completes the installation, reboot the machine.")
 
     util.prompt_for_enter()
 
-    util._print_sub_header("Host Requirements")
+    util.print_sub_header("Host Requirements")
 
-
-    print("Now, install all pre-requisites (in the virtual machine) by running:")
+    print("Next, install all the software pre-requisites by running:")
 
     print("""
     blocksat-cli deps install
@@ -155,37 +161,37 @@ def _print_usb_rx_instructions(info):
     _print("""
     NOTE: this command supports the apt, dnf, and yum package managers. For
     other package managers, refer to the instructions at:""")
-    print("https://github.com/Blockstream/satellite/blob/master/doc/tbs.md")
+    print(defs.user_guide_url + "doc/tbs.html")
 
     util.prompt_for_enter()
 
-    util._print_sub_header("Configure the Host")
+    util.print_sub_header("Configure the Host")
 
-    _print("""Next, you need to create and configure the network interfaces that
-    will output the IP traffic received via the TBS5927. You can apply all
-    configurations by running the following command:""")
-
-    print("\n    sudo blocksat-cli usb config\n")
-
-    _print("""If you would like to review the changes that will be made before
-    applying them, first run the command as a non-root user:
-    """)
+    _print("""Next, you need to create and configure the network interface that
+    will output the IP traffic received via the {} device. You can apply all
+    configurations by running the following command:""".format(name))
 
     print("\n    blocksat-cli usb config\n")
 
-    _print("""
-    Note this command will define arbitrary IP addresses to the interfaces. If
-    you need (or want) to define specific IP addresses instead, for example to
-    avoid IP address conflicts, use command-line argument `--ip`.
+    _print("""If you would like to review the changes before applying them,
+    first run the command in dry-run mode:
     """)
 
-    _print("""Furthermore, note that this configuration is not persistent across
-    reboots. After a reboot, you need to run `sudo blocksat-cli usb config`
+    print("\n    blocksat-cli usb config --dry-run\n")
+
+    _print("""
+    Note this command will define an arbitrary IP address to the interface. If
+    you would like to set a specific IP address, for example, to avoid
+    address conflicts, use the option \"--ip\".
+    """)
+
+    _print("""Furthermore, note this configuration is not persistent across
+    reboots. After a reboot, you need to run \"blocksat-cli usb config\"
     again.""")
 
     util.prompt_for_enter()
 
-    util._print_sub_header("Launch")
+    util.print_sub_header("Launch")
 
     print("Finally, start the receiver by running:")
 
@@ -197,9 +203,9 @@ def _print_usb_rx_instructions(info):
 def _print_sdr_instructions(info):
     """Print instruction for configuration of an SDR setup
     """
-    util._print_header("SDR Setup")
+    util.print_header("SDR Setup")
 
-    util._print_sub_header("Connections")
+    util.print_sub_header("Connections")
 
     print("The SDR setup is connected as follows:\n")
 
@@ -220,12 +226,13 @@ def _print_sdr_instructions(info):
 
     util.prompt_for_enter()
 
-    util._print_sub_header("Software Requirements")
+    util.print_sub_header("Software Requirements")
 
     print("The SDR-based setup relies on the applications listed below:\n")
 
     _item("leandvb: a software-based DVB-S2 receiver application.")
-    _item("rtl_sdr: reads samples taken by the RTL-SDR and feeds them into leandvb.")
+    _item("rtl_sdr: reads samples taken by the RTL-SDR and feeds them into "
+          "leandvb.")
     _item("TSDuck: unpacks the output of leandvb and produces "
           "IP packets to be fed to Bitcoin Satellite.")
     _item("Gqrx: useful for spectrum visualization during antenna pointing.")
@@ -235,43 +242,41 @@ def _print_sdr_instructions(info):
     blocksat-cli deps install
     """)
 
-    _print(
-        """
+    _print("""
         NOTE: This command supports the two most recent Ubuntu LTS, Fedora, and
         CentOS releases. In case you are using another Linux distribution or
         version, please refer to the manual compilation and installation
         instructions at:""")
-    print("https://github.com/Blockstream/satellite/blob/master/doc/sdr.md")
+    print(defs.user_guide_url + "doc/sdr.html")
 
     util.prompt_for_enter()
 
-    util._print_sub_header("Configuration")
+    util.print_sub_header("Configuration")
 
-    _print("Next, you can generate the configurations that are needed for gqrx "
-           "by running:")
+    _print(
+        "Next, you can generate the configurations that are needed for gqrx "
+        "by running:")
 
     print("    blocksat-cli gqrx-conf")
 
     util.prompt_for_enter()
 
-    util._print_sub_header("Running")
+    util.print_sub_header("Running")
 
-    _print("You should now be ready to launch the SDR receiver. You can run it "
-           "by executing:")
+    _print(
+        "You should now be ready to launch the SDR receiver. You can run it "
+        "by executing:")
 
     print("    blocksat-cli sdr\n")
     print("Or, in GUI mode:\n")
     print("    blocksat-cli sdr --gui\n")
 
-    _print("For further options, please refer to the SDR Guide at:")
-    print("https://github.com/Blockstream/satellite/blob/master/doc/sdr.md")
-
     util.prompt_for_enter()
 
 
 def _print_sat_ip_instructions(info):
 
-    util._print_header("Sat-IP Setup")
+    util.print_header("Sat-IP Setup")
 
     _print("""The Sat-IP setup relies on the Blockstream Satellite Base
     Station (available on Blockstream Store), an all-in-one flat-panel antenna
@@ -284,7 +289,7 @@ def _print_sat_ip_instructions(info):
 
     util.prompt_for_enter()
 
-    util._print_sub_header("Connections")
+    util.print_sub_header("Connections")
 
     _item("Connect the Ethernet cable from your switch or computer's network "
           "adapter directly to the antenna's Sat>IP port.")
@@ -302,7 +307,7 @@ def _print_sat_ip_instructions(info):
 
     util.prompt_for_enter()
 
-    util._print_sub_header("Software Requirements")
+    util.print_sub_header("Software Requirements")
 
     _print("Next, install all software pre-requisites on your host. Run:")
 
@@ -313,7 +318,7 @@ def _print_sat_ip_instructions(info):
 
     util.prompt_for_enter()
 
-    util._print_sub_header("Running")
+    util.print_sub_header("Running")
 
     _print("You should now be ready to launch the Sat-IP client. You can run "
            "it by executing:")
@@ -325,92 +330,101 @@ def _print_sat_ip_instructions(info):
 
 def _print_freq_info(info):
     """Print summary of frequencies of interest"""
-    sat     = info['sat']
-    setup   = info['setup']
-    lnb     = info['lnb']
+    sat = info['sat']
+    setup = info['setup']
+    lnb = info['lnb']
     lo_freq = info['freqs']['lo']
-    l_freq  = info['freqs']['l_band']
+    l_freq = info['freqs']['l_band']
 
-    util._print_header("Frequencies")
+    util.print_header("Frequencies")
 
-    print("For your information, your setup relies on the following frequencies:\n")
-    print("| Downlink %2s band frequency            | %8.2f MHz |" %(sat['band'], sat['dl_freq']))
-    print("| LNB local oscillator (LO) frequency   | %8.2f MHz |" %(lo_freq))
-    print("| Receiver L-band frequency             | %7.2f MHz  |" %(l_freq))
+    print("For your information, your setup relies on the following "
+          "frequencies:\n")
+    print("| Downlink %2s band frequency            | %8.2f MHz |" %
+          (sat['band'], sat['dl_freq']))
+    print("| LNB local oscillator (LO) frequency   | %8.2f MHz |" % (lo_freq))
+    print("| Receiver L-band frequency             | %7.2f MHz  |" % (l_freq))
     print()
 
     if (lnb['universal'] and (setup['type'] == defs.sdr_setup_type)):
         if (sat['dl_freq'] > defs.ku_band_thresh):
             print("NOTE regarding Universal LNB:\n")
 
-            print(textwrap.fill(("The DL frequency of {} is in Ku high "
-                                 "band (> {:.1f} MHz). Hence, you need to use "
-                                 "the higher frequency LO ({:.1f} MHz) of your "
-                                 "Universal LNB. This requires a 22 kHz tone "
-                                 "to be sent to the LNB."
-            ).format(sat['alias'], defs.ku_band_thresh, lo_freq)))
+            print(
+                textwrap.fill(
+                    ("The DL frequency of {} is in Ku high "
+                     "band (> {:.1f} MHz). Hence, you need to use "
+                     "the higher frequency LO ({:.1f} MHz) of your "
+                     "Universal LNB. This requires a 22 kHz tone "
+                     "to be sent to the LNB.").format(sat['alias'],
+                                                      defs.ku_band_thresh,
+                                                      lo_freq)))
             print()
-            print(textwrap.fill(("With a software-defined setup, you will "
-                                 "need to place a 22 kHz tone generator "
-                                 "inline between the LNB and the power "
-                                 "inserter. Typically the tone generator "
-                                 "uses power from the power inserter while "
-                                 "delivering the tone directly to the "
-                                 "LNB.")))
+            print(
+                textwrap.fill(("With a software-defined setup, you will "
+                               "need to place a 22 kHz tone generator "
+                               "inline between the LNB and the power "
+                               "inserter. Typically the tone generator "
+                               "uses power from the power inserter while "
+                               "delivering the tone directly to the "
+                               "LNB.")))
 
     util.prompt_for_enter()
 
 
 def _print_lnb_info(info):
     """Print important waraning based on LNB choice"""
-    lnb   = info['lnb']
-    sat   = info['sat']
+    lnb = info['lnb']
+    sat = info['sat']
     setup = info['setup']
 
     if ((lnb['pol'] != "Dual") and (lnb['pol'] != sat['pol'])):
-        util._print_header("LNB Information")
+        util.print_header("LNB Information")
         lnb_pol = "Vertical" if lnb['pol'] == "V" else "Horizontal"
-        logging.warning(textwrap.fill(
-            "Your LNB has {} polarization and the signal from {} has the "
-            "opposite polarization.".format(lnb_pol, sat['name'])))
+        logging.warning(
+            textwrap.fill(
+                "Your LNB has {} polarization and the signal from {} has the "
+                "opposite polarization.".format(lnb_pol, sat['name'])))
         util.prompt_for_enter()
 
     if ((lnb['pol'] == "Dual") and (setup['type'] == defs.sdr_setup_type)):
-        util._print_header("LNB Information")
-        logging.warning(textwrap.fill(
-            "Your LNB has dual polarization. Check the voltage of your power "
-            "supply in order to discover the polarization on which your LNB "
-            "will operate."))
+        util.print_header("LNB Information")
+        logging.warning(
+            textwrap.fill(
+                "Your LNB has dual polarization. Check the voltage of your "
+                "power supply in order to discover the polarization on which "
+                "your LNB will operate."))
         util.prompt_for_enter()
 
 
 def _print_next_steps():
-    util._print_header("Next Steps")
+    util.print_header("Next Steps")
     _print("""
-    At this point, if your dish is already correctly pointed, you should be able to
-    start receiving data on Bitcoin Satellite.
+    At this point, if your dish is already correctly pointed, you should be
+    able to start receiving data on Bitcoin Satellite.
     """)
 
-    print("You can generate a bitcoin.conf configuration file for Bitcoin Satellite using:")
+    print("You can generate a bitcoin.conf configuration file for Bitcoin "
+          "Satellite using:")
     print("\n    blocksat-cli btc\n")
 
-
-    _print("Next, if you are running Ubuntu, Fedora, or CentOS, you can "
-           "install bitcoin-satellite by running:")
+    _print("Next, if you are running a supported Linux distribution "
+           "(Ubuntu, Debian, Fedora, CentOS, or Raspberry Pi OS), you can "
+           "install Bitcoin Satellite by running:")
     print("    blocksat-cli deps install --btc\n")
 
-    _print("Note that bitcoin-satellite is a fork of bitcoin core, and, "
+    _print("Note that Bitcoin Satellite is a fork of Bitcoin Core, and, "
            "as such, it installs applications with the same name (bitcoind, "
            "bitcoin-cli, bitcoin-qt, and bitcoin-tx). Hence, the "
-           "bitcoin-satellite installation will fail if you already have "
-           "bitcoin core installed.")
+           "Bitcoin Satellite installation will fail if you already have "
+           "Bitcoin Core installed.")
 
     print("For further information, refer to:\n")
-    print("https://github.com/Blockstream/satellite/blob/master/doc/bitcoin.md\n")
+    print(defs.user_guide_url + "doc/bitcoin.html\n")
 
     _print("""If your antenna is not pointed yet, please follow the
     antenna alignment guide at:""")
-    print("https://github.com/Blockstream/satellite/blob/master/doc/antenna-pointing.md\n")
+    print(defs.user_guide_url + "doc/antenna-pointing.html\n")
 
 
 def subparser(subparsers):
