@@ -1,9 +1,15 @@
 """SDR Receiver Wrapper"""
-import subprocess, logging, textwrap, os, threading, time
+import logging
+import subprocess
+import textwrap
+import threading
 from argparse import ArgumentDefaultsHelpFormatter
 from shutil import which
+
 from . import config, defs, util, dependencies, monitoring, tsp
+
 logger = logging.getLogger(__name__)
+runner = util.ProcessRunner(logger)
 
 
 def _tune_max_pipe_size(pipesize):
@@ -17,38 +23,39 @@ def _tune_max_pipe_size(pipesize):
     current_max = int(ret.decode().split()[-1])
 
     if (current_max < pipesize):
-        cmd = util.root_cmd(["sysctl", "-w",
-                             "fs.pipe-max-size=" + str(pipesize)])
-
-        print(textwrap.fill("The maximum pipe size that is currently "
-                            "configured in your OS is of {} bytes, which is "
-                            "not sufficient for the receiver application. "
-                            "It will be necessary to run the following command "
-                            "as root:".format(current_max), width=80))
+        cmd = ["sysctl", "-w", "fs.pipe-max-size=" + str(pipesize)]
+        print(
+            textwrap.fill("The maximum pipe size that is currently "
+                          "configured in your OS is of {} bytes, which is "
+                          "not sufficient for the receiver application. "
+                          "It will be necessary to run the following command "
+                          "as root:".format(current_max),
+                          width=80))
         print("\n" + " ".join(cmd) + "\n")
 
-        if (not util._ask_yes_or_no("Is that OK?", default="y")):
+        if (not util.ask_yes_or_no("Is that OK?", default="y")):
             print("Abort")
             return False
 
-        return subprocess.check_output(cmd)
+        res = runner.run(cmd, root=True)
+        return (res.returncode == 0)
     else:
         return True
 
 
 # Rx status metrics printed by leandvb
 rx_stat_map = {
-    'SS' : { # based on the AGC but not calibrated to dBm (unitless)
-        'key'  : 'level',
-        'unit' : None
+    'SS': {  # based on the AGC but not calibrated to dBm (unitless)
+        'key': 'level',
+        'unit': None
     },
-    'MER' : {
-        'key'  : 'snr',
-        'unit' : 'dB'
+    'MER': {
+        'key': 'snr',
+        'unit': 'dB'
     },
-    'VBER' : { # measured based on the BCH decoder output
-        'key'  : 'ber',
-        'unit' : None
+    'VBER': {  # measured based on the BCH decoder output
+        'key': 'ber',
+        'unit': None
     }
 }
 
@@ -68,18 +75,16 @@ def _get_monitor(args):
     # Force scrolling logs if tsp is configured to print to stdout
     scrolling = tsp.prints_to_stdout(args) or args.log_scrolling
 
-    return monitoring.Monitor(
-        args.cfg_dir,
-        logfile = args.log_file,
-        scroll = scrolling,
-        echo = echo,
-        min_interval = args.log_interval,
-        server = args.monitoring_server,
-        port = args.monitoring_port,
-        report = args.report,
-        report_opts = monitoring.get_report_opts(args),
-        utc = args.utc
-    )
+    return monitoring.Monitor(args.cfg_dir,
+                              logfile=args.log_file,
+                              scroll=scrolling,
+                              echo=echo,
+                              min_interval=args.log_interval,
+                              server=args.monitoring_server,
+                              port=args.monitoring_port,
+                              report=args.report,
+                              report_opts=monitoring.get_report_opts(args),
+                              utc=args.utc)
 
 
 def _monitor_demod_info(info_pipe, monitor):
@@ -96,15 +101,10 @@ def _monitor_demod_info(info_pipe, monitor):
                     monitoring and logging
 
     """
-    assert(isinstance(info_pipe, util.Pipe))
+    assert (isinstance(info_pipe, util.Pipe))
 
     # "Standard" status format accepted by the Monitor class
-    status = {
-        'lock': (False, None),
-        'level': None,
-        'snr': None,
-        'ber': None
-    }
+    status = {'lock': (False, None), 'level': None, 'snr': None, 'ber': None}
 
     while True:
         line = info_pipe.readline()
@@ -114,9 +114,9 @@ def _monitor_demod_info(info_pipe, monitor):
 
         for metric in rx_stat_map:
             if metric in line:
-                val         = float(line.split()[-1])
-                unit        = rx_stat_map[metric]['unit']
-                key         = rx_stat_map[metric]['key']
+                val = float(line.split()[-1])
+                unit = rx_stat_map[metric]['unit']
+                key = rx_stat_map[metric]['key']
                 status[key] = (val, unit)
 
         # If unlocked, clear the status metrics that depend on receiver locking
@@ -149,70 +149,114 @@ def subparser(subparsers):
     monitoring.add_to_parser(p)
 
     rtl_p = p.add_argument_group('rtl_sdr options')
-    rtl_p.add_argument('--sps', default=2.0, type=float,
+    rtl_p.add_argument('--sps',
+                       default=2.0,
+                       type=float,
                        help='Samples per symbol, or, equivalently, the '
-                           'target oversampling ratio')
-    rtl_p.add_argument('--rtl-idx', default=0, type=int,
+                       'target oversampling ratio')
+    rtl_p.add_argument('--rtl-idx',
+                       default=0,
+                       type=int,
                        help='RTL-SDR device index')
-    rtl_p.add_argument('-g', '--gain', default=40, type=float,
+    rtl_p.add_argument('-g',
+                       '--gain',
+                       default=40,
+                       type=float,
                        help='RTL-SDR Rx gain')
-    rtl_p.add_argument('-f', '--iq-file', default=None,
+    rtl_p.add_argument('-f',
+                       '--iq-file',
+                       default=None,
                        help='File to read IQ samples from instead of reading '
                        'from the RTL-SDR in real-time')
 
     ldvb_p = p.add_argument_group('leandvb options')
-    ldvb_p.add_argument('-n', '--n-helpers', default=6, type=int,
-                        help='Number of instances of the external LDPC decoder \
+    ldvb_p.add_argument(
+        '-n',
+        '--n-helpers',
+        default=6,
+        type=int,
+        help='Number of instances of the external LDPC decoder \
                         to spawn as child processes')
-    ldvb_p.add_argument('-d', '--debug-dvbs2', action='count', default=0,
-                        help="Debug leandvb's DVB-S2 decoding. Use it multiple "
-                        "times to increase the debugging level")
-    ldvb_p.add_argument('-v', '--verbose', default=False, action='store_true',
+    ldvb_p.add_argument(
+        '-d',
+        '--debug-dvbs2',
+        action='count',
+        default=0,
+        help="Debug leandvb's DVB-S2 decoding. Use it multiple "
+        "times to increase the debugging level")
+    ldvb_p.add_argument('-v',
+                        '--verbose',
+                        default=False,
+                        action='store_true',
                         help='leandvb in verbose mode')
-    ldvb_p.add_argument('--gui', default=False, action='store_true',
+    ldvb_p.add_argument('--gui',
+                        default=False,
+                        action='store_true',
                         help='GUI mode')
-    ldvb_p.add_argument('--derotate', default=0, type=float,
+    ldvb_p.add_argument('--derotate',
+                        default=0,
+                        type=float,
                         help='Frequency offset correction to apply in kHz')
-    ldvb_p.add_argument('--fastlock', default=False, action='store_true',
+    ldvb_p.add_argument('--fastlock',
+                        default=False,
+                        action='store_true',
                         help='leandvb fast lock mode')
-    ldvb_p.add_argument('--rrc-rej', default=30, type=int,
+    ldvb_p.add_argument('--rrc-rej',
+                        default=30,
+                        type=int,
                         help='leandvb RRC rej parameter')
-    ldvb_p.add_argument('-m', '--modcod', choices=defs.modcods.keys(),
-                        default='qpsk3/5', metavar='',
+    ldvb_p.add_argument('-m',
+                        '--modcod',
+                        choices=defs.modcods.keys(),
+                        default='qpsk3/5',
+                        metavar='',
                         help="DVB-S2 modulation and coding (MODCOD) scheme. "
                         "Choose from: " + ", ".join(defs.modcods.keys()))
-    ldvb_p.add_argument('--ldpc-dec', default="ext", choices=["int", "ext"],
+    ldvb_p.add_argument('--ldpc-dec',
+                        default="ext",
+                        choices=["int", "ext"],
                         help="LDPC decoder to use (internal or external)")
-    ldvb_p.add_argument('--ldpc-bf', default=100,
+    ldvb_p.add_argument('--ldpc-bf',
+                        default=100,
                         help='Max number of iterations used by the internal \
                         LDPC decoder when not using an external LDPC tool')
-    ldvb_p.add_argument('--ldpc-iterations', default=25,
+    ldvb_p.add_argument('--ldpc-iterations',
+                        default=25,
                         help='Max number of iterations used by the external \
                         LDPC decoder when using an external LDPC tool')
-    ldvb_p.add_argument('--framesizes', type=int, default=1, choices=[0,1,2,3],
+    ldvb_p.add_argument('--framesizes',
+                        type=int,
+                        default=1,
+                        choices=[0, 1, 2, 3],
                         help="Bitmask of desired frame sizes (1=normal, \
                         2=short)")
-    ldvb_p.add_argument('--no-tsp', default=False, action='store_true',
+    ldvb_p.add_argument('--no-tsp',
+                        default=False,
+                        action='store_true',
                         help='Feed leandvb output to stdout instead of tsp')
-    ldvb_p.add_argument('--pipe-size', default=32, type=int,
+    ldvb_p.add_argument('--pipe-size',
+                        default=32,
+                        type=int,
                         help='Size in Mbytes of the input pipe file read by \
                         leandvb')
 
     # TSDuck Options
     tsp.add_to_parser(p)
 
-    p.set_defaults(func=run,
-                   record=False)
+    p.set_defaults(func=run, record=False)
 
     subsubparsers = p.add_subparsers(title='subcommands',
                                      help='Target sub-command')
     # IQ recording
-    p2 = subsubparsers.add_parser('rec',
-                                  description="Record IQ samples instead of "
-                                  "feeding them into leandvb",
-                                  help='Record IQ samples',
-                                  formatter_class=ArgumentDefaultsHelpFormatter)
-    p2.add_argument('-f', '--iq-file', default="blocksat.iq",
+    p2 = subsubparsers.add_parser(
+        'rec',
+        description="Record IQ samples instead of "
+        "feeding them into leandvb",
+        help='Record IQ samples',
+        formatter_class=ArgumentDefaultsHelpFormatter)
+    p2.add_argument('-f',
+                    '--iq-file',
+                    default="blocksat.iq",
                     help='File on which to save IQ samples received with '
                     'the RTL-SDR.')
     p2.set_defaults(record=True)
@@ -226,6 +270,8 @@ def run(args):
     if (info is None):
         return
 
+    util.check_configured_setup_type(info, defs.sdr_setup_type, logger)
+
     pipe_size_bytes = int(args.pipe_size * (2**20))
     if (not _tune_max_pipe_size(pipe_size_bytes)):
         return
@@ -240,9 +286,9 @@ def run(args):
         return
 
     # Receiver configs
-    l_band_freq = info['freqs']['l_band']*1e6
+    l_band_freq = info['freqs']['l_band'] * 1e6
     sym_rate = defs.sym_rate[info['sat']['alias']]
-    samp_rate = args.sps*sym_rate
+    samp_rate = args.sps * sym_rate
     modcod = str(2**defs.modcods[args.modcod])  # bit mask
 
     assert(samp_rate < 2.4e6), \
@@ -252,42 +298,55 @@ def run(args):
     # derotation exceeds the CFO correction range, change the L-band frequency
     # directly on the RTL instead.
     cfo_corr_range = (sym_rate / 4)
-    derotate       = args.derotate*1e3
+    derotate = args.derotate * 1e3
     while (derotate > cfo_corr_range):
-        derotate    -= cfo_corr_range
+        derotate -= cfo_corr_range
         l_band_freq += cfo_corr_range
     while (derotate < -cfo_corr_range):
-        derotate    += cfo_corr_range
+        derotate += cfo_corr_range
         l_band_freq -= cfo_corr_range
 
     if (args.iq_file is None or args.record):
-        rtl_cmd = ["rtl_sdr", "-g", str(args.gain), "-f", str(l_band_freq),
-                   "-s", str(samp_rate), "-d", str(args.rtl_idx)]
+        rtl_cmd = [
+            "rtl_sdr", "-g",
+            str(args.gain), "-f",
+            str(l_band_freq), "-s",
+            str(samp_rate), "-d",
+            str(args.rtl_idx)
+        ]
         if (args.record):
-            bytes_per_sec = (2*samp_rate)/(2**20)
+            bytes_per_sec = (2 * samp_rate) / (2**20)
             logger.info("IQ recording will be saved on file {}".format(
                 args.iq_file))
             logger.info(
                 textwrap.fill("NOTE: the file will grow by approximately "
                               "{} MB per second.".format(bytes_per_sec)))
-            if (not util._ask_yes_or_no("Proceed?", default="y")):
+            if (not util.ask_yes_or_no("Proceed?", default="y")):
                 return
             rtl_cmd.append(args.iq_file)
         else:
             rtl_cmd.append("-")
 
-    ldvb_cmd = ["leandvb", "--nhelpers", str(args.n_helpers), "-f",
-                str(samp_rate), "--sr", str(sym_rate), "--roll-off",
-                str(defs.rolloff), "--standard", "DVB-S2", "--sampler", "rrc",
-                "--rrc-rej", str(args.rrc_rej), "--modcods", modcod,
-                "--framesizes", str(args.framesizes)]
+    ldvb_cmd = [
+        "leandvb", "--nhelpers",
+        str(args.n_helpers), "-f",
+        str(samp_rate), "--sr",
+        str(sym_rate), "--roll-off",
+        str(defs.rolloff), "--standard", "DVB-S2", "--sampler", "rrc",
+        "--rrc-rej",
+        str(args.rrc_rej), "--modcods", modcod, "--framesizes",
+        str(args.framesizes)
+    ]
     if (args.iq_file is None):
         ldvb_cmd.extend(["--inpipe", str(pipe_size_bytes)])
     if (args.debug_dvbs2 > 0):
         ldvb_cmd.extend(("-d " * args.debug_dvbs2).split())
     if (args.ldpc_dec == "ext"):
-        ldvb_cmd.extend(["--ldpc-helper", which("ldpc_tool"),
-                         "--ldpc-iterations", str(args.ldpc_iterations)])
+        ldvb_cmd.extend([
+            "--ldpc-helper",
+            which("ldpc_tool"), "--ldpc-iterations",
+            str(args.ldpc_iterations)
+        ])
     else:
         ldvb_cmd.extend(["--ldpc-bf", str(args.ldpc_bf)])
     if (args.gui):
@@ -316,16 +375,14 @@ def run(args):
     # leandvb (via --fd-info). Collect this information on a daemon thread.
     if (args.iq_file is None):
         info_pipe = util.Pipe()
-        fd_info   = str(info_pipe.w_fd)
+        fd_info = str(info_pipe.w_fd)
         ldvb_cmd.extend(["--fd-info", fd_info])
 
         monitor = _get_monitor(args)
 
-        t = threading.Thread(
-            target = _monitor_demod_info,
-            args   = (info_pipe, monitor),
-            daemon = True
-        )
+        t = threading.Thread(target=_monitor_demod_info,
+                             args=(info_pipe, monitor),
+                             daemon=True)
         t.start()
 
     # Prepare and validate the tsp command
@@ -334,25 +391,21 @@ def run(args):
         return
 
     if (args.iq_file is None):
-        full_cmd  = "> " + " ".join(rtl_cmd) + " | \\\n" + \
+        full_cmd = "> " + " ".join(rtl_cmd) + " | \\\n" + \
                     " ".join(ldvb_cmd)
         p1 = subprocess.Popen(rtl_cmd, stdout=subprocess.PIPE)
-        p2 = subprocess.Popen(
-            ldvb_cmd,
-            stdin=p1.stdout,
-            stdout=subprocess.PIPE,
-            stderr=ldvb_stderr,
-            pass_fds=[fd_info]
-        )
+        p2 = subprocess.Popen(ldvb_cmd,
+                              stdin=p1.stdout,
+                              stdout=subprocess.PIPE,
+                              stderr=ldvb_stderr,
+                              pass_fds=[fd_info])
     else:
-        full_cmd   = "> " + " ".join(ldvb_cmd) + " < " + args.iq_file
+        full_cmd = "> " + " ".join(ldvb_cmd) + " < " + args.iq_file
         fd_iq_file = open(args.iq_file)
-        p2 = subprocess.Popen(
-            ldvb_cmd,
-            stdin=fd_iq_file,
-            stdout=subprocess.PIPE,
-            stderr=ldvb_stderr
-        )
+        p2 = subprocess.Popen(ldvb_cmd,
+                              stdin=fd_iq_file,
+                              stdout=subprocess.PIPE,
+                              stderr=ldvb_stderr)
 
     if (not args.no_tsp):
         full_cmd += " | \\\n" + " ".join(tsp_handler.cmd)
