@@ -3,6 +3,7 @@ from . import bitcoin, defs, config
 
 
 class TestBitcoinConfGen(unittest.TestCase):
+
     def gen_text_dict(self):
         """Generate configurations in text and dictionary formats"""
         # Text version
@@ -10,25 +11,24 @@ class TestBitcoinConfGen(unittest.TestCase):
             "debug=udpnet\n"
             "debug=udpmulticast\n"
             "udpmulticastloginterval=60\n"
-            "udpmulticast=dvb0_0,239.0.0.2:4434,172.16.235.9,1,blocksat-tbs-lowspeed\n"
-            "udpmulticast=dvb0_1,239.0.0.2:4434,172.16.235.9,1,blocksat-tbs-highspeed\n"
+            "udpmulticast=dvb0_0,239.0.0.2:4434,172.16.235.9,1,tbs-lowspeed\n"
+            "udpmulticast=dvb0_1,239.0.0.2:4434,172.16.235.9,1,tbs-highspeed\n"
         )
         # Corresponding dictionary version
         udpmcastlist = list()
-        devices      = ["dvb0_0", "dvb0_1"]
-        labels       = ["blocksat-tbs-lowspeed", "blocksat-tbs-highspeed"]
-        for device,label in zip(devices, labels):
+        devices = ["dvb0_0", "dvb0_1"]
+        labels = ["tbs-lowspeed", "tbs-highspeed"]
+        for device, label in zip(devices, labels):
             udpmcastlist.append(
                 bitcoin._udpmulticast(device,
                                       "172.16.235.9",
                                       dst_addr=defs.btc_dst_addr,
                                       trusted="1",
-                                      label=label)
-            )
+                                      label=label))
         cfg = {
-            'debug'                   : ["udpnet", "udpmulticast"],
-            'udpmulticastloginterval' : "60",
-            'udpmulticast'            : udpmcastlist
+            'debug': ["udpnet", "udpmulticast"],
+            'udpmulticastloginterval': "60",
+            'udpmulticast': udpmcastlist
         }
         return text, cfg
 
@@ -58,15 +58,15 @@ class TestBitcoinConfGen(unittest.TestCase):
         # Create configuration object with the given configs
         cfg = bitcoin.Cfg(dct)
         # Add new option
-        cfg.add_opt("udpmulticast",
-                    bitcoin._udpmulticast("eth0",
-                                          "172.16.235.9",
-                                          dst_addr=defs.btc_dst_addr,
-                                          trusted="1",
-                                          label="blocksat-s400")
-                    )
+        cfg.add_opt(
+            "udpmulticast",
+            bitcoin._udpmulticast("eth0",
+                                  "172.16.235.9",
+                                  dst_addr=defs.btc_dst_addr,
+                                  trusted="1",
+                                  label="s400"))
         # Add new option to expected text
-        text += "udpmulticast=eth0,239.0.0.2:4434,172.16.235.9,1,blocksat-s400\n"
+        text += "udpmulticast=eth0,239.0.0.2:4434,172.16.235.9,1,s400\n"
         # Check if exported text includes the new option
         self.assertEqual(text, cfg.text())
 
@@ -84,18 +84,18 @@ class TestBitcoinConfGen(unittest.TestCase):
         self.assertTrue('en0,239.0.0.2:4434,172.16.235.1,' in opt)
 
         # USB Rx
-        info['setup'] = defs.demods[1]
-        ifname = config.get_net_if(info)
-        cfg = bitcoin._gen_cfgs(info, ifname)
-        opt = cfg.cfg['udpmulticast']
-        self.assertTrue('dvb0_0,239.0.0.2:4434,172.16.235.1,' in opt)
+        for demod in [1, 2]:
+            info['setup'] = defs.demods[demod]
+            ifname = config.get_net_if(info)
+            cfg = bitcoin._gen_cfgs(info, ifname)
+            opt = cfg.cfg['udpmulticast']
+            self.assertTrue('dvb0_0,239.0.0.2:4434,172.16.235.1,' in opt)
 
         # SDR and Sat-IP Rx (both use the loopback interface and the 127.0.0.1
         # source address)
-        for demod in defs.demods[2:4]:
-            info['setup'] = demod
+        for demod in [3, 4]:
+            info['setup'] = defs.demods[demod]
             ifname = config.get_net_if(info)
             cfg = bitcoin._gen_cfgs(info, ifname)
             opt = cfg.cfg['udpmulticast']
             self.assertTrue('lo,239.0.0.2:4434,127.0.0.1,' in opt)
-
